//Written for www.HomoFaciens.de
//Program demonstrates effects of static and kinetic friction at a drag race.
//Copyright (C) 2010 Norbert Heinz
//
//This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free //Software Foundation version 3 of the License.
//
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or //FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.

import java.awt.*;
import java.awt.event.*;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import java.awt.image.*;
import java.io.*;
import java.net.URL;
import javax.imageio.ImageIO;
import javax.swing.*;
import java.text.*;
import java.util.GregorianCalendar;
import java.util.Calendar;
import javax.sound.sampled.*;
import java.applet.*;
import java.util.Hashtable;
import javax.swing.event.*;





public class staticfriction extends JApplet implements  ActionListener, ChangeListener  {

    static String versionNumber="v1.0";

    ScenePainter painter;
    GraphPainter graph01;
    int sceneHeight=200, sceneWidth=900;
    Box dummyBoxCol1 = Box.createHorizontalBox(); //Empty boxes for creating fixed row width and column height
    Box dummyBoxCol2 = Box.createHorizontalBox();
    Box dummyBoxCol3 = Box.createHorizontalBox();
    Box dummyBoxRow1 = Box.createHorizontalBox();
    Box dummyBoxRow2 = Box.createHorizontalBox();
    Box dummyBoxRow3 = Box.createHorizontalBox();
    JSpinner frictionSpinner;
    JSlider throttlePedalSlider, opponentSkillSlider, frictionSlider;
    SpinnerNumberModel powerModel;
    JButton calcButton;
    JLabel powerLabel, throttlePedalLabel, opponentSkillLabel;
    JLabel powerUnitLabel, frictionLabel;
    JLabel sourceLabel, versionLabel;
    int[][] xKoords;          //keeps values for 5 graphs
    int graphActive=0;        //painted graphnumber for simulation (0-4)
    double xMax=1.0, yMax=10.0;  //maximum values for graph
    double coordsCount=350.0; //number of coordinates to calculate for graphpainting
    int iPos[];        //x-endposition in Pixel while simulation is running
    int iPosNow=0, iPosOld=0;
    boolean isRunning=false; //is simulation still running?
    Timer timer;             //timer for simulation
    int timerPause=25;       //delay between two calculated frames in milliseconds
    String xValue="", yValue="";  //Labeling of x- and y-axis
    long simulationTime, startTime, simulationTimeOld;  //times passed while simulation is running
    String[] resultString, formularString;  //calculation results painted for each graph
    double coordMulti=0.0;
    static String language="initalValue";
    String givenParam="nothing yet";   //Arguments passed to applet
    boolean firstPaint=false;          //clears and repaints bufferimage for graphs if true
    double doveSpeed=15.0;
    double truckAcceleration=1.25;
    double doveCycle;         //Wingposition while dove is flying
    String speedName, timeName, accelerationName, startName, stopName, minName, maxName, throttlePedalName;
    String opponentMinName, opponentMaxName, opponentSkillName, powerName, frictionName, frictionMinName, frictionMaxName;
    String hint01Name, hint02Name;
    String truckImageName, carImageName, guidePostImageName, backGroundImageName, truckTrailerImageName, sunImageName, moonImageName;
    String carTireImageName, truckTireImageName, truckTireDrivingImageName, smoke01ImageName, smoke02ImageName, ImageName, tireBatchImageName;
    String signalGreenImageName, signalYellowImageName, signalRedImageName, carTireDrivingImageName;
    String doveWingImageName, doveImageName;
    String doveSoundWaveName, carSoundWaveName, truckSoundWaveName, tireSquealingWaveName;
    Font font = new Font("Arial", Font.PLAIN, 12);
    double raceDistance=402.336, carDistance=0.0;

    public void init() {
        UIManager.put("swing.boldMetal", Boolean.FALSE);

        try{
          givenParam = getParameter("LanguageSettings");
        }
        catch(NullPointerException npe){
          System.out.println("could not get parameter 'LanguageSettings'");
        }

        if(givenParam.compareTo("nothing yet")!=0) {
          if (givenParam.compareTo("German")==0)language="German";
          else language="English";
        }
    }

    public void start() {
      int i, i2;

        initComponents();

        xKoords=new int[5][(int)coordsCount];
        iPos=new int[5];
        timer = new Timer(timerPause, new timerEvent());
        resultString = new String[5];
        formularString = new String[5];
        for(i=0;i<5;i++){
          resultString[i]="";
          formularString[i]="";
          iPos[i]=0;
          for(i2=0;i2<coordsCount;i2++){
            xKoords[i][i2]=0;
          }
        }
        graphActive=-1;
    }

    public static void main(String[] args) {
        String appName="Static friction";

        if(args.length>0){
          if (args[0].compareTo("German")==0){
            language="German";
            appName="Haft- und Gleitreibung";
          }
          else{
            language="English";
            appName="Static and kinetic friction";
          }
        }
        else{
          language="English";
          appName="Static and kinetic friction";
        }

        JFrame f = new JFrame(appName);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JApplet ap = new staticfriction();
        ap.init();
        ap.start();
        f.getContentPane().add("Center", ap);
        f.pack();
        f.setVisible(true);

    }


    private BufferedImage loadImage(String name) {
        String imgFileName = "images/"+name;
        URL url = staticfriction.class.getResource(imgFileName);
        BufferedImage img = null;
        try {
            img =  ImageIO.read(url);
        }
        catch (Exception e){
          img=null;
          System.out.println("Error loading Image '" + imgFileName + "': " + e); // Display the string.
        }
        return img;
    }

    private AudioClip loadSound(String name) {
        String soundFileName = "sounds/"+name;
        AudioClip audioClp=null;
        try {
          URL url = this.getClass().getClassLoader().getResource(soundFileName);
          audioClp = newAudioClip(url);
        }
        catch (Exception e){
          audioClp=null;
          System.out.println("Error loading Sound '" + soundFileName + "': " + e); // Display the string.
        }
        return audioClp;
    }

    public void setSpinnerWidth() {
        JFormattedTextField ftf = null;
        ftf = getTextField(frictionSpinner);
        if (ftf != null ) {
            ftf.setFont(font);
            ftf.setColumns(6); //specify more width than we need
            ftf.setHorizontalAlignment(JTextField.RIGHT);
        }
    }

    public void initComponents() {
        int i;
        GridBagConstraints c = new GridBagConstraints();

        getContentPane().setLayout(new BorderLayout());

        //create panels
        JPanel controlPanel = new JPanel();
        JPanel scenePanel = new JPanel();
        JPanel mainPane = new JPanel();
        JPanel graphPanel = new JPanel();
        JPanel throttlePedalPanel = new JPanel();

        //set panel properties
        controlPanel.setLayout(new GridBagLayout());
        throttlePedalPanel.setLayout(new GridBagLayout());
        mainPane.setLayout(new BoxLayout(mainPane, BoxLayout.PAGE_AXIS));
        mainPane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        if(language.compareTo("German")==0){
          timeName="Zeit";
          powerName="Leistung";
          speedName="Geschwindigkeit";
          accelerationName="Beschleunigung";
          startName="Start";
          stopName="Stop";
          minName="Min";
          maxName="Max";
          throttlePedalName="Gaspedal";
          opponentMinName="Anfänger";
          opponentMaxName="Profi";
          opponentSkillName="Stärke Gegner";
          frictionName="Straßenbelag";
          frictionMinName="Rutschig";
          frictionMaxName="Griffig";
          hint01Name="Verschiebe mit der Maus das Gaspedal zum Beschleunigen !!!";
          hint02Name="Durchdrehende Reifen verhindern optimales Beschleunigen !!!";

          carImageName = "fiat126.png";
          carTireImageName = "fiat126-tire.png";
          carTireDrivingImageName = "fiat126-tire-driving.png";
          smoke01ImageName = "smoke01.png";
          smoke02ImageName = "smoke02.png";
          guidePostImageName = "guidepost.png";
          truckImageName = "truck.png";
          truckTireImageName = "truck-tire.png";
          truckTireDrivingImageName = "truck-tire-driving.png";
          truckTrailerImageName = "trailer.png";
          backGroundImageName = "background.png";
          sunImageName = "sun.png";
          moonImageName = "moon.png";
          doveImageName = "dove.png";
          doveWingImageName = "dovewing.png";
          signalRedImageName = "signal_light_red.png";
          signalYellowImageName = "signal_light_yellow.png";
          signalGreenImageName = "signal_light_green.png";
          tireBatchImageName = "tires.png";

          truckSoundWaveName = "truck.wav";
          carSoundWaveName = "car.wav";
          doveSoundWaveName = "dove.wav";
          tireSquealingWaveName = "squealing.wav";

        }
        else if(language.compareTo("XXX")==0){
          timeName="XXX";
        }
        else{
          timeName="Time";
          powerName="Power";
          speedName="Speed";
          accelerationName="Acceleration";
          startName="Start";
          stopName="Stop";
          minName="Min";
          maxName="Max";
          throttlePedalName="Throttle pedal";
          opponentMinName="Rookie";
          opponentMaxName="Pro";
          opponentSkillName="Opponent skill";
          frictionName="Grip level";
          frictionMinName="Slippery";
          frictionMaxName="Maximum grip";
          hint01Name="Move throttle pedal with your mouse to accelerate !!!";
          hint02Name="No burnouts for optimal acceleration !!!";

          carImageName = "fiat126.png";
          carTireImageName = "fiat126-tire.png";
          carTireDrivingImageName = "fiat126-tire-driving.png";
          smoke01ImageName = "smoke01.png";
          smoke02ImageName = "smoke02.png";
          guidePostImageName = "guidepost.png";
          truckImageName = "truck.png";
          truckTireImageName = "truck-tire.png";
          truckTireDrivingImageName = "truck-tire-driving.png";
          truckTrailerImageName = "trailer.png";
          backGroundImageName = "background_en.png";
          sunImageName = "sun.png";
          moonImageName = "moon.png";
          doveImageName = "dove.png";
          doveWingImageName = "dovewing.png";
          signalRedImageName = "signal_light_red.png";
          signalYellowImageName = "signal_light_yellow.png";
          signalGreenImageName = "signal_light_green.png";
          tireBatchImageName = "tires.png";

          truckSoundWaveName = "truck.wav";
          carSoundWaveName = "car.wav";
          doveSoundWaveName = "dove.wav";
          tireSquealingWaveName = "squealing.wav";

        }

        //create buttons, labels etcetera
        throttlePedalSlider = new JSlider(JSlider.VERTICAL, 0, 1000, 0);
        opponentSkillSlider = new JSlider(JSlider.HORIZONTAL, 0, 1000, 777);
        frictionSlider = new JSlider(JSlider.HORIZONTAL, 0, 1000, 700);

        calcButton = new JButton(startName);

        powerLabel = new JLabel(powerName + ":");
        throttlePedalLabel = new JLabel(throttlePedalName);
        opponentSkillLabel = new JLabel(opponentSkillName + ":  77%");
        frictionLabel = new JLabel(frictionName + ":  0.475");

        powerUnitLabel = new JLabel("kW");

        xValue="s [m]";
        yValue="v [m/s]";

        sourceLabel = new JLabel("Source & info:    www.HomoFaciens.de");
        versionLabel = new JLabel(versionNumber);

        powerModel = new SpinnerNumberModel(17 , 1, 300, 1);

        frictionSpinner = new JSpinner(powerModel);
        setSpinnerWidth();

        painter = new ScenePainter();
        graph01 = new GraphPainter();

        //set properties of controls
        painter.hint01Name=hint01Name;
        painter.hint02Name=hint02Name;

        Hashtable throttleLabelTable =  new Hashtable();
        throttleLabelTable.put(new Integer( 0 ), new JLabel(minName) );
        throttleLabelTable.put(new Integer( 1000 ), new JLabel(maxName) );
        throttlePedalSlider.setLabelTable(throttleLabelTable);
        throttlePedalSlider.setPaintLabels(true);
        throttlePedalSlider.setFont(font);

        Hashtable opponentLabelTable =  new Hashtable();
        opponentLabelTable.put(new Integer( 0 ), new JLabel(opponentMinName) );
        opponentLabelTable.put(new Integer( 1000 ), new JLabel(opponentMaxName) );
        opponentSkillSlider.setLabelTable(opponentLabelTable);
        opponentSkillSlider.setPaintLabels(true);
        opponentSkillSlider.setFont(font);
        opponentSkillSlider.addChangeListener(this);


        Hashtable frictionLabelTable =  new Hashtable();
        frictionLabelTable.put(new Integer( 0 ), new JLabel(frictionMinName) );
        frictionLabelTable.put(new Integer( 1000 ), new JLabel(frictionMaxName) );
        frictionSlider.setLabelTable(frictionLabelTable);
        frictionSlider.setPaintLabels(true);
        frictionSlider.setFont(font);
        frictionSlider.addChangeListener(this);

        powerUnitLabel.setFont(font);

        powerLabel.setFont(font);

        sourceLabel.setFont(font);
        versionLabel.setFont(font);

        calcButton.setFont(font);
        calcButton.setActionCommand("calcButtonPressed");
        calcButton.addActionListener(this);

        painter.car = loadImage(carImageName);
        painter.carTire = loadImage(carTireImageName);
        painter.carTireDriving = loadImage(carTireDrivingImageName);
        painter.smoke01 = loadImage(smoke01ImageName);
        painter.smoke02 = loadImage(smoke02ImageName);
        painter.guidePost = loadImage(guidePostImageName);
        painter.truck = loadImage(truckImageName);
        painter.truckTire = loadImage(truckTireImageName);
        painter.truckTireDriving = loadImage(truckTireDrivingImageName);
        painter.truckTrailer = loadImage(truckTrailerImageName);
        painter.backGround = loadImage(backGroundImageName);
        painter.sun = loadImage(sunImageName);
        painter.moon = loadImage(moonImageName);
        painter.dove = loadImage(doveImageName);
        painter.doveWing = loadImage(doveWingImageName);
        painter.signalRed = loadImage(signalRedImageName);
        painter.signalYellow = loadImage(signalYellowImageName);
        painter.signalGreen = loadImage(signalGreenImageName);
        painter.tireBatch = loadImage(tireBatchImageName);
        painter.signalX=painter.sceneWidth/2.0-250;

        painter.truckSound=loadSound(truckSoundWaveName);
        painter.carSound=loadSound(carSoundWaveName);
        painter.doveSound=loadSound(doveSoundWaveName);
        painter.tireSound=loadSound(tireSquealingWaveName);

        painter.doveY=(Math.random() * painter.sceneHeight * 0.7);
        painter.truckStartX=(painter.sceneWidth-painter.car.getWidth(null))/2.0;
        painter.doveStartX=painter.sceneWidth;
        doveCycle=0.0;
        painter.signalStartX=painter.sceneWidth/2.0-250.0;

        painter.guidePostX=0.0;
        painter.doveX=painter.doveStartX;
        painter.truckX=painter.truckStartX;
        painter.trailerX=painter.truckX;
        painter.signalX=painter.signalStartX;
        painter.backGroundX=0.0;
        painter.tireBatchX=0.0;


        //add components
        c.fill = GridBagConstraints.HORIZONTAL;
        c.ipadx = 10;
        c.insets = new Insets(5,5,5,5);  //padding

        dummyBoxCol1.add(dummyBoxCol1.createRigidArea(new Dimension (130,1)));
        dummyBoxCol3.add(dummyBoxCol3.createRigidArea(new Dimension (30,1)));
        dummyBoxRow1.add(dummyBoxRow1.createRigidArea(new Dimension (1,20)));
        dummyBoxRow2.add(dummyBoxRow2.createRigidArea(new Dimension (1,20)));
        dummyBoxRow3.add(dummyBoxRow3.createRigidArea(new Dimension (1,20)));

        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor=GridBagConstraints.CENTER;


        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth=3;
        controlPanel.add(opponentSkillLabel, c);

        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth=3;
        controlPanel.add(opponentSkillSlider, c);

        c.gridx = 0;
        c.gridy = 3;
        c.gridwidth=3;
        controlPanel.add(frictionLabel, c);

        c.gridx = 0;
        c.gridy = 4;
        c.gridwidth=3;
        controlPanel.add(frictionSlider, c);

        c.gridx = 0;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(powerLabel, c);

        c.gridx = 1;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(frictionSpinner, c);

        c.gridx = 2;
        c.gridy = 5;
        c.gridwidth=1;
        controlPanel.add(powerUnitLabel, c);

        c.gridx = 0;
        c.gridy = 9;
        c.gridwidth=3;
        controlPanel.add(calcButton, c);

        //set dummyBoxes for fixed layout of columns and rows
        c.gridwidth=1;
        c.gridx = 3;
        c.gridy = 4;
        controlPanel.add(dummyBoxRow1, c);
        c.gridx = 3;
        c.gridy = 5;
        controlPanel.add(dummyBoxRow2, c);
        c.gridx = 0;
        c.gridy = 10;
        controlPanel.add(dummyBoxCol1, c);
        c.gridx = 2;
        c.gridy = 10;
        controlPanel.add(dummyBoxCol3, c);

        c.gridx = 0;
        c.gridy = 11;
        c.gridwidth=3;
        c.fill = GridBagConstraints.NONE;
        c.anchor=GridBagConstraints.LINE_END;
        controlPanel.add(sourceLabel, c);

        c.gridx = 0;
        c.gridy = 12;
        c.gridwidth=3;
        c.fill = GridBagConstraints.NONE;
        c.anchor=GridBagConstraints.LINE_END;
        controlPanel.add(versionLabel, c);

        dummyBoxCol2.add(dummyBoxCol2.createRigidArea(new Dimension (100,1)));
        c.gridx = 0;
        c.gridy = 0;
        c.anchor=GridBagConstraints.CENTER;
        throttlePedalPanel.add(dummyBoxCol2, c);

        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth=1;
        throttlePedalPanel.add(throttlePedalLabel, c);

        c.gridx = 0;
        c.gridy = 2;
        throttlePedalPanel.add(throttlePedalSlider, c);

        //setup panels
        scenePanel.add("Center", painter);
        graphPanel.add("Center", graph01);
        graphPanel.add("Center", throttlePedalPanel);
        graphPanel.add("Center", controlPanel);

        mainPane.add(scenePanel);
        mainPane.add(graphPanel);
        getContentPane().add(mainPane);
        getRootPane().setDefaultButton(calcButton);

    }

    public JFormattedTextField getTextField(JSpinner spinner) {
        JComponent editor = spinner.getEditor();
        if (editor instanceof JSpinner.DefaultEditor) {
            return ((JSpinner.DefaultEditor)editor).getTextField();
        } else {
            System.err.println("Unexpected editor type: "  + spinner.getEditor().getClass() + " isn't a descendant of DefaultEditor");
            return null;
        }
    }

    public void resetValues() {
        int i, i2;
        for(i=0;i<5;i++){
          resultString[i]="";
          formularString[i]="";
          iPos[i]=0;
        }
        graphActive=-1;
        xMax=1.0;
        yMax=1.0;
    }

    public void stateChanged(ChangeEvent e) {
      opponentSkillLabel.setText(opponentSkillName + ":  " + new DecimalFormat("###").format(opponentSkillSlider.getValue()/10) + "%");
      frictionLabel.setText(frictionName + ":  " + new DecimalFormat("0.000").format(frictionSlider.getValue() * 0.65 / 1000.0 + 0.02));
    }

    // Listens to the buttons.
    public void actionPerformed(ActionEvent e) {
        int i, i2;

        if("calcButtonPressed".equals(e.getActionCommand())) {
          if(painter.truckSound != null){
            painter.truckSound.stop();
            painter.truckSoundIsRunning=false;
          }
          if(painter.doveSound != null){
            painter.doveSound.stop();
            painter.doveSoundIsRunning=false;
          }

          if(isRunning){
            if(painter.carSound != null){
              painter.carSound.stop();
              painter.carSoundIsRunning=false;
            }
            calcButton.setText(startName);
            isRunning=false;
            painter.isRunning=false;
          }
          else{
            iPosNow=0;
            iPosOld=0;
            firstPaint=true;
            calcButton.setText(stopName);
            graphActive++;
            if(graphActive>4)graphActive=0;
            startTime=System.currentTimeMillis();
            simulationTime=0;
            simulationTimeOld=0;

            frictionSpinner.setEnabled(false);
            opponentSkillSlider.setEnabled(false);
            frictionSlider.setEnabled(false);
            painter.carSpeed=0.0;
            painter.truckSpeed=0.0;
            carDistance=0.0;
            throttlePedalSlider.setValue(0);
            painter.isSlipping=false;
            painter.trailerLoosened=false;
            painter.tireSoundIsRunning=false;

            xMax=raceDistance;
            coordMulti = (double)graph01.graphHeight / yMax;
            painter.truckDistance=0.0;

            formularString[graphActive] = "???";

            if(timer.getInitialDelay()!=timerPause){
              timer.setInitialDelay(timerPause);
            }
            timer.start();
            isRunning=true;
            if(painter.doveX!=painter.doveStartX)painter.doveY=(Math.random() * painter.sceneHeight * 0.7);
            painter.doveStartX=painter.sceneWidth;
            doveCycle=0.0;
            painter.truckStartX=(painter.sceneWidth-painter.car.getWidth(null))/2.0;
            painter.signalStartX=painter.sceneWidth/2.0-250.0;

            painter.guidePostX=0.0;     //Return car to starting line
            painter.doveX=painter.doveStartX;
            painter.truckX=painter.truckStartX;
            painter.trailerX=painter.truckX;
            painter.signalX=painter.signalStartX;
            painter.backGroundX=0.0;
            painter.tireBatchX=0.0;
            painter.signalStatus=0;      //reset status of traffic light
          }//if(!isRunning)
        }//if ("calcButtonPressed".equals(e.getActionCommand())) {
    }

    private class timerEvent implements ActionListener{
      public void actionPerformed(ActionEvent e){
        int i, i2;
        double timeDiff;

        simulationTimeOld=simulationTime;
        simulationTime=System.currentTimeMillis()-startTime;
        timeDiff=(double)(simulationTime-simulationTimeOld)/1000.0;
        if(timeDiff<=0.0 || timeDiff>1000.0){//This should normaly NOT happen !?!?
          //System.err.println("Unusual value for timeDiff: "  + timeDiff);
          timeDiff=0.000001;
        }

        if(carDistance>raceDistance | !isRunning){//End simulation
          timer.stop();
          formularString[graphActive]= new DecimalFormat("0.0").format((double)simulationTime/1000.0) + "s";
          frictionSpinner.setEnabled(true);
          opponentSkillSlider.setEnabled(true);
          frictionSlider.setEnabled(true);
          isRunning=false;
          painter.isRunning=false;
          if(painter.truckSound != null){
            painter.truckSound.stop();
          }
          if(painter.carSound != null){
            painter.carSound.stop();
            painter.carSoundIsRunning=false;
          }
          if(painter.doveSound != null){
            painter.doveSound.stop();
          }
          if(painter.tireSound != null){
            painter.tireSound.stop();
            painter.tireSoundIsRunning=false;
          }
          calcButton.setText(startName);
          graph01.refreshScene(xKoords, iPos, xValue, yValue, resultString, formularString, xMax, yMax, 999);//paint all lines (999)
        }

        //car is 3meters long, picture of car is 195pixels wide: 195.0/3.0 is the number of pixels for 1meter.
        if(isRunning){
          if(painter.signalStatus>1){//Traffic light is green
            double carAcceleration;
            double carWeight=800.0, carPower=0.0, truckPower=0.0;
            double v1, v2;
            double gripLevel=0.5;

            gripLevel = frictionSlider.getValue() * 0.65 / 1000.0 + 0.02;
            carPower = powerModel.getNumber().doubleValue() * throttlePedalSlider.getValue();
            //Acceleration is calculated by given power and kinetic energy of the car
            v1=painter.carSpeed;
            v2=Math.sqrt(Math.pow(v1, 2.0) + 2 * carPower * timeDiff / carWeight);
            carAcceleration = (v2 - v1) / timeDiff;

            if(powerModel.getNumber().doubleValue()>100.0){
              painter.trailerLoosened=true;
            }

            //truck has same weight and power as car
            truckPower = powerModel.getNumber().doubleValue()*1000.0;
            //Acceleration is calculated by given power and kinetic energy of the truck
            v1=painter.truckSpeed;
            v2=Math.sqrt(Math.pow(v1, 2.0) + 2 * truckPower * timeDiff / carWeight);
            truckAcceleration = (v2 - v1) / timeDiff;
            //Truck is never exceeding grip level and so never sliding
            if(truckAcceleration > gripLevel * 9.81){
              truckAcceleration = gripLevel * 9.81;
            }
            //Depending on value of opponentSkillSlider, truck acceleration is reduced
            truckAcceleration *= opponentSkillSlider.getValue() / 1000.0;
            painter.truckSpeed += truckAcceleration * timeDiff;

            //Reducing acceleration of car if tires are slipping
            if(!painter.isSlipping){
              if(carAcceleration > gripLevel * 9.81){
                painter.isSlipping = true;
                carAcceleration = gripLevel * 0.4 * 9.81;
              }
            }
            else{
              //Stop slipping of tires if acceleration is lower than acceleration with slipping tires
              if(carAcceleration < gripLevel * 0.4 * 9.81){
                painter.isSlipping = false;
              }
              else{
                carAcceleration = gripLevel * 0.4 * 9.81;
              }
            }
            painter.carSpeed += carAcceleration * timeDiff;
            carDistance += painter.carSpeed * timeDiff + carAcceleration / 2.0 * Math.pow(timeDiff, 2.0);
            painter.guidePostX = carDistance * 195.0 / 3.0;
            painter.signalX = painter.signalStartX + carDistance * 195.0 / 3.0;

            painter.truckDistance+=painter.truckSpeed * timeDiff + truckAcceleration/2.0*Math.pow(timeDiff, 2.0);
            painter.truckX = (carDistance - painter.truckDistance) * 195.0 / 3.0;
            painter.truckX += painter.truckStartX;
            if(!painter.trailerLoosened){
              painter.trailerX = painter.truckX;
            }
            else{
              painter.trailerX = painter.truckStartX + carDistance * 195.0 / 3.0;
            }

            painter.doveX=(carDistance -(simulationTime/1000.0)*doveSpeed)*195.0/3.0;
            doveCycle+=timeDiff*1000.0/350.0;
            painter.doveX+=painter.doveStartX;
            painter.doveWingHeight=(int) (59.0 * Math.sin(doveCycle));

            painter.backGroundX=carDistance*195.0/3.0/3.0;
            painter.tireBatchX=carDistance*195.0/3.0;
            while(painter.backGroundX>painter.sceneWidth)painter.backGroundX-=sceneWidth;
            while(painter.tireBatchX>painter.sceneWidth)painter.tireBatchX-=sceneWidth;

            if(painter.carSpeed > yMax){
              coordMulti = (double)graph01.graphHeight / (painter.carSpeed * 1.2);
              for(i2=0;i2<5;i2++){
                for(i=0;i<coordsCount;i++){
                  xKoords[i2][i]= (int) (xKoords[i2][i] * yMax / (painter.carSpeed * 1.2));
                }
              }
              yMax = painter.carSpeed * 1.2;
              graph01.refreshScene(xKoords, iPos, xValue, yValue, resultString, formularString, xMax, yMax, 999);//Don't paint active graph
              graph01.addLine(iPosNow, xKoords[graphActive][iPosNow], -1);
            }

            iPosOld=iPosNow;
            iPosNow=(int)(carDistance*(coordsCount-1)/raceDistance);
            if(iPosNow>=coordsCount)iPosNow=(int)coordsCount-1;
            iPos[graphActive]=iPosNow;

            for(i=iPosOld;i<=iPosNow;i++){
              xKoords[graphActive][i]=(int)(painter.carSpeed * coordMulti);
            }

            if(iPosNow>0 && iPosNow<coordsCount){
              graph01.addLine(iPosNow, xKoords[graphActive][iPosNow], graphActive);
            }

            if(isRunning){
              painter.isRunning=true;
              if(painter.carSound!=null && !painter.carSoundIsRunning){
                painter.carSound.loop();
                painter.carSoundIsRunning=true;
              }
            }
          }
          else{//traffic light turns from red to green
            if(simulationTime>1000){
              startTime=System.currentTimeMillis();
              simulationTime=0;
              painter.signalStatus++;
            }
          }
        }//if(isRunning)
        if(firstPaint){//Graph is refreshed
          graph01.refreshScene(xKoords, iPos, xValue, yValue, resultString, formularString, xMax, yMax, graphActive);//Don't paint active graph
          graph01.xAlt=0;
          graph01.yAlt=xKoords[graphActive][0];
          firstPaint=false;
        }
        painter.refreshScene();
      }
    }
}



class ScenePainter extends Component {

    BufferedImage car = null, smoke01 = null, smoke02 = null, guidePost = null, truck = null, carTire=null, carTireDriving = null, backGround = null;
    BufferedImage truckTire = null, truckTireDriving = null, truckTrailer = null, tireBatch = null;
    BufferedImage sun = null, moon = null, dove = null, doveWing = null;
    BufferedImage signalRed = null, signalYellow = null, signalGreen = null;
    BufferedImage bufferImage=null;//buffer for the whole scene painting to avoid flickering
    int doveWingHeight=10;
    int sceneHeight = 200, sceneWidth = 900;
    int i;
    double guidePostX=0.0, truckX=1000.0, trailerX=1000.0, backGroundX=0.0, doveX=-1000.0, doveY=10.0, tireBatchX=0.0;
    double manX=-100.0, manSize=100.0, manCycle=0.0, manCycle2=0.0, manY=100.0, sscX=2000.0;
    double truckStartX=0.0, truckDistance=0.0, doveStartX=0.0, signalStartX=0.0;
    double signalX=0.0;
    int[] tireHub={0,0};  //makes the tires of the car jump while driving
    int[] truckTireHub={0, 0, 0, 0, 0};
    int tireMaxHub=1, truckTireMaxHub=2;
    Graphics2D bufferImageSurface=null;
    AudioClip truckSound=null, carSound=null, doveSound=null, tireSound = null;
    boolean truckSoundIsRunning=false, carSoundIsRunning=false, doveSoundIsRunning=false, tireSoundIsRunning;
    boolean isRunning=false;
    boolean truckSoundStopped=true, carSoundStopped=true, doveSoundStopped=true;
    double carSpeed=0.0, truckSpeed=0.0;
    int signalStatus=0;
    boolean isSlipping=false, trailerLoosened=false;;
    String hint01Name="", hint02Name="";

    public Dimension getPreferredSize(){
        return new Dimension(sceneWidth, sceneHeight);
    }

    void refreshScene() {

      //play sound, if truck, dove, man etcetera reach middle of the scene (=position of car)
      if(truckSound!=null){
        if(truckX<sceneWidth/2.0 + 20.0 && truckX>sceneWidth/2.0 - 20.0){
          if (!truckSoundIsRunning && isRunning){
            truckSound.play();
            truckSoundIsRunning=true;
            truckSoundStopped=false;
          }
        }
        else{
          truckSoundIsRunning=false;
        }
        if((truckX<0.0 || truckX>sceneWidth) && !truckSoundStopped){
          truckSound.stop();
          truckSoundIsRunning=false;
          truckSoundStopped=true;
        }
      }
      if(doveSound!=null){
        if(doveX<sceneWidth/2.0 + 20.0 && doveX>sceneWidth/2.0 - 20.0){
          if (!doveSoundIsRunning && isRunning){
            doveSound.play();
            doveSoundIsRunning=true;
            doveSoundStopped=false;
          }
        }
        else{
          doveSoundIsRunning=false;
        }
        if((doveX<0.0 || doveX>sceneWidth) && !doveSoundStopped){
          doveSound.stop();
          doveSoundIsRunning=false;
          doveSoundStopped=true;
        }
      }
      while((int) guidePostX > (double) (sceneWidth) )guidePostX -= 195.0 * 50.0 / 3.0;

      //tires of car jump higher, the faster the car is driving
      tireMaxHub=1;
      if(carSpeed>1.0)tireMaxHub=1;
      if(carSpeed>10.0)tireMaxHub=3;
      if(carSpeed>30.0)tireMaxHub=4;
      if(carSpeed>50.0)tireMaxHub=5;
      repaint();
    }


    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Dimension size = getSize();
        Composite origComposite;
        int sunX, sunY;
        boolean sunShine=false;
        Calendar cal = new GregorianCalendar();
        int secondsSinceMidnight = cal.get(Calendar.HOUR_OF_DAY) * 3600 + cal.get(Calendar.MINUTE) * 60 + cal.get(Calendar.SECOND);

        //create buffer to avoid flickering while scene is calculated and painted
        if(bufferImage==null){
          bufferImage = (BufferedImage)createImage(sceneWidth, sceneHeight);
          bufferImageSurface = bufferImage.createGraphics();
        }

        origComposite = g2.getComposite();

        //calculates if sun (6:00 - 18:00) or moon (18:00 - 6:00) is shining
        //sunX: sun or moon are moving horizontal
        //suny: sun rising from 6:00 to 12:00 and declines from 12:00 - 18:00
        //      moon rising from 18:00 to 00:00 and declines from 00:00 - 6:00
        if(secondsSinceMidnight>6*3600 && secondsSinceMidnight<18*3600){//Sun shining
          secondsSinceMidnight-=6*3600;
          sunShine=true;
          sunX=(int)(sceneWidth * 0.1 + sceneWidth * 0.8 * secondsSinceMidnight / (3600.0 * 12.0));
          if(secondsSinceMidnight>6*3600){
            sunY=(int)( sceneHeight * 0.4 * ((secondsSinceMidnight - 3600.0 * 6.0) / (3600.0 * 6.0)));
          }
          else{
            sunY=(int)( sceneHeight * 0.4 * (1.0 - (secondsSinceMidnight / (3600.0 * 6.0))));
          }
        }
        else{//moon shining
          if(secondsSinceMidnight>18*3600){
            sunShine=false;
            secondsSinceMidnight-=12*3600;
          }
          if(secondsSinceMidnight>6*3600){
            sunX=(int)(sceneWidth * 0.5 - sceneWidth * 0.8 * (1.0 - secondsSinceMidnight / (3600.0 * 12.0)));
            sunY=(int)( sceneHeight * 0.4 * (1.0- (secondsSinceMidnight - 3600.0 * 6.0) / (3600.0 * 6.0)));
          }
          else{
            sunX=(int)(sceneWidth * 0.5 + sceneWidth * 0.8 * secondsSinceMidnight / (3600.0 * 12.0));
            sunY=(int)( sceneHeight * 0.4 * (secondsSinceMidnight / (3600.0 * 6.0)));
          }
        }
        //center of sun or moon sets position, not the left corner of the image
        if(sunShine && sun != null)sunX-=sun.getWidth(null)/2;
        if(!sunShine && moon != null)sunX-=moon.getWidth(null)/2;

        if(sunShine){//light blue background while sun is shining
          bufferImageSurface.setBackground(new Color(223, 252, 255)); 
        }
        else{//black background during night
          bufferImageSurface.setBackground(new Color(0, 0, 50)); 
        }
        bufferImageSurface.clearRect(0, 0, sceneWidth, sceneHeight);

        if(sunShine){
          if(sun != null)bufferImageSurface.drawImage(sun, sunX, sunY, 100, 100, this);
        }
        else{
          if(moon != null)bufferImageSurface.drawImage(moon, sunX, sunY, 100, 100, this);
        }

        if(backGround!=null){
          bufferImageSurface.drawImage(backGround, (int)backGroundX, 0, sceneWidth, sceneHeight, this);
          bufferImageSurface.drawImage(backGround, (int)backGroundX-sceneWidth, 0, sceneWidth, sceneHeight, this);
        }
        if(tireBatch!=null){
          bufferImageSurface.drawImage(tireBatch, (int)tireBatchX, sceneHeight-tireBatch.getHeight(null), this);
          bufferImageSurface.drawImage(tireBatch, (int)tireBatchX-sceneWidth, sceneHeight-tireBatch.getHeight(null), this);
        }
        if(guidePost != null)bufferImageSurface.drawImage(guidePost, (int) (guidePostX), sceneHeight-guidePost.getHeight(null), this);


        if(truck != null && truckTireDriving != null && truckTire != null){
          if(trailerX<sceneWidth && trailerX>-truck.getWidth(null)-400.0){
            if(truckTrailer != null){
              bufferImageSurface.drawImage(truckTrailer, (int) (trailerX+110.0), sceneHeight-truckTrailer.getHeight(null), this);
            }
            if(isRunning){//let the tires jump while truck is driving
              if(!trailerLoosened){
                bufferImageSurface.drawImage(truckTireDriving, (int)truckX+355, sceneHeight-truckTireDriving.getHeight(null)-(int) truckTireHub[3], this);
                bufferImageSurface.drawImage(truckTireDriving, (int)truckX+415, sceneHeight-truckTireDriving.getHeight(null)-(int) truckTireHub[4], this);
              }
              else{
                bufferImageSurface.drawImage(truckTire, (int)trailerX+355, sceneHeight-truckTireDriving.getHeight(null), this);
                bufferImageSurface.drawImage(truckTire, (int)trailerX+415, sceneHeight-truckTireDriving.getHeight(null), this);
              }
            }
            else{
              bufferImageSurface.drawImage(truckTire, (int)trailerX+355, sceneHeight-truckTireDriving.getHeight(null), this);
              bufferImageSurface.drawImage(truckTire, (int)trailerX+415, sceneHeight-truckTireDriving.getHeight(null), this);
            }
          }
          if(truckX<sceneWidth && truckX>-truck.getWidth(null)-400.0){//paint truck
            bufferImageSurface.drawImage(truck, (int) (truckX), sceneHeight-truck.getHeight(null), this);
            if(isRunning){//let the tires jump while truck is driving
              for(i=0;i<5;i++){
                truckTireHub[i]=(int)(Math.random() * (double)truckTireMaxHub);
              }
              bufferImageSurface.drawImage(truckTireDriving, (int)truckX+10, sceneHeight-truckTireDriving.getHeight(null)-(int) truckTireHub[0], this);
              bufferImageSurface.drawImage(truckTireDriving, (int)truckX+115, sceneHeight-truckTireDriving.getHeight(null)-(int) truckTireHub[1], this);
              bufferImageSurface.drawImage(truckTireDriving, (int)truckX+172, sceneHeight-truckTireDriving.getHeight(null)-(int) truckTireHub[2], this);
            }
            else{
              bufferImageSurface.drawImage(truckTire, (int)truckX+10, sceneHeight-truckTireDriving.getHeight(null), this);
              bufferImageSurface.drawImage(truckTire, (int)truckX+115, sceneHeight-truckTireDriving.getHeight(null), this);
              bufferImageSurface.drawImage(truckTire, (int)truckX+172, sceneHeight-truckTireDriving.getHeight(null), this);
            }
          }
        }

        if(car != null)bufferImageSurface.drawImage(car,(sceneWidth-car.getWidth(null))/2, sceneHeight-car.getHeight(null), this);
        if(isRunning && carSpeed>0.0){
          for(i=0;i<2;i++){
            tireHub[i]=(int)(Math.random() * (double)tireMaxHub);
          }
          if(carTireDriving != null){//let the tires jump while car is driving
            bufferImageSurface.drawImage(carTireDriving,(sceneWidth-carTireDriving.getWidth(null))/2-68, sceneHeight-carTireDriving.getHeight(null)-(int)tireHub[0], this);
            bufferImageSurface.drawImage(carTireDriving,(sceneWidth-carTireDriving.getWidth(null))/2+55, sceneHeight-carTireDriving.getHeight(null)-(int)tireHub[1], this);
          }
        }
        else{
          if(carTire != null){
            bufferImageSurface.drawImage(carTire,(sceneWidth-carTire.getWidth(null))/2-68, sceneHeight-carTire.getHeight(null), this);
            bufferImageSurface.drawImage(carTire,(sceneWidth-carTire.getWidth(null))/2+55, sceneHeight-carTire.getHeight(null), this);
          }
        }

        if(isSlipping){//Paint smoke while tire is slipping
          if(smoke01 != null && smoke02 != null && car != null){
            if(Math.random()>0.5){
              bufferImageSurface.drawImage(smoke01,sceneWidth/2+59, sceneHeight-smoke01.getHeight(null), this);
            }
            else{
              bufferImageSurface.drawImage(smoke02,sceneWidth/2+59, sceneHeight-smoke01.getHeight(null), this);
            }
          }
          if(!tireSoundIsRunning && isRunning){
            if(tireSound != null){
              tireSound.loop();
              tireSoundIsRunning=true;
            }
          }
        }
        else{
          if(tireSoundIsRunning){
            tireSound.stop();
            tireSoundIsRunning = false;
          }
        }

        if(signalX<sceneWidth){
          if(signalStatus==0){
            if(signalRed != null)bufferImageSurface.drawImage(signalRed, (int)signalX, sceneHeight-signalRed.getHeight(null), this);
          }
          if(signalStatus==1){
            if(signalYellow != null)bufferImageSurface.drawImage(signalYellow, (int)signalX, sceneHeight-signalRed.getHeight(null), this);
          }
          if(signalStatus>1){
            if(signalGreen != null)bufferImageSurface.drawImage(signalGreen, (int)signalX, sceneHeight-signalRed.getHeight(null), this);
          }
        }

        if(dove != null && doveWing != null){
          if(doveX<sceneWidth && doveX>-dove.getWidth(null)){//paint dove
            bufferImageSurface.drawImage(dove, (int)doveX, (int)doveY, this);
            bufferImageSurface.drawImage(doveWing, (int)doveX+20, (int)doveY+15-doveWingHeight, doveWing.getWidth(null), (int)doveWingHeight, this);
          }
        }

        //Give hints how to drive the car
        if(carSpeed==0.0 && signalStatus>1){
          bufferImageSurface.setColor(new Color(255, 255, 0));
          bufferImageSurface.fillRect(90, 0, 600, 40);
          bufferImageSurface.setColor(new Color(255, 0, 100));
          Font font = new Font("Arial", Font.PLAIN, 20);
          bufferImageSurface.setFont(font);
          bufferImageSurface.drawString(hint01Name, 100, 30);
        }
        if(isSlipping && signalStatus>1){
          bufferImageSurface.setColor(new Color(255, 255, 0));
          bufferImageSurface.fillRect(90, 0, 750, 40);
          bufferImageSurface.setColor(new Color(255, 0, 100));
          Font font = new Font("Arial", Font.PLAIN, 20);
          bufferImageSurface.setFont(font);
          bufferImageSurface.drawString(hint02Name, 100, 30);
        }

        bufferImageSurface.setColor(new Color(0, 0, 0));
        bufferImageSurface.drawRect(0, 0, sceneWidth-1, sceneHeight-1);

        //paint bufferImage and make it visible
        g2.drawImage(bufferImage, 0, 0, this);

        g2.setComposite(origComposite);

    }
}

class GraphPainter extends Component {
    int graphWidth=350, graphHeight=220;
    Color[] colorUsed= {new Color(200, 0, 0), new Color(0, 200, 0), new Color(0, 0, 200), new Color(0, 200, 200), new Color(200, 0, 200)};
    BufferedImage bufferImage=null;
    Graphics2D bufferImageSurface=null;
    int xAlt, yAlt;

    public Dimension getPreferredSize(){
        return new Dimension(graphWidth + 50 + 50, graphHeight + 55);
    }

    void addLine(int xNeu, int yNeu, int plotColor){
      if(plotColor<0){
        xAlt=xNeu;
        yAlt=yNeu;
      }
      else{
        if((xNeu != xAlt || yNeu != yAlt) && plotColor < 5){
          bufferImageSurface.setColor(colorUsed[plotColor]);
          bufferImageSurface.drawLine(xAlt+49, graphHeight-yAlt+20, xNeu+50, graphHeight-yNeu+20);
          xAlt=xNeu;
          yAlt=yNeu;
          repaint();
        }
      }
    }

    void refreshScene(int[][] x,
                      int[] iPos,
                      String xString,
                      String yString,
                      String[] rString,
                      String[] fString,
                      double xMVal,
                      double yMVal,
                      int dontPaint){
        int i, i2;

        if(bufferImage==null){
          bufferImage = (BufferedImage)createImage(graphWidth + 150, graphHeight + 55);
          bufferImageSurface = bufferImage.createGraphics();
          bufferImageSurface.setBackground(new Color(238, 238, 238));
        }

        bufferImageSurface.clearRect(0, 0, graphWidth + 250, graphHeight + 55);

        bufferImageSurface.setColor(Color.BLACK);
        //Y-Axis
        bufferImageSurface.drawLine(50, 20, 50, graphHeight+30);
        bufferImageSurface.drawLine(50, 20, 40, 30);
        bufferImageSurface.drawLine(50, 20, 60, 30);
        bufferImageSurface.drawLine(40, 20+(int)(graphHeight*0.25), 50, 20+(int)(graphHeight*0.25));
        bufferImageSurface.drawLine(40, 20+(int)(graphHeight*0.5), 50, 20+(int)(graphHeight*0.5));
        bufferImageSurface.drawLine(40, 20+(int)(graphHeight*0.75), 50, 20+(int)(graphHeight*0.75));
        //X-Axis
        bufferImageSurface.drawLine(40, graphHeight+20, graphWidth+50, graphHeight+20);
        bufferImageSurface.drawLine(graphWidth+40, graphHeight+10, graphWidth+50, graphHeight+20);
        bufferImageSurface.drawLine(graphWidth+40, graphHeight+30, graphWidth+50, graphHeight+20);
        bufferImageSurface.drawLine(50+(int)(graphWidth*0.25), graphHeight+20, 50+(int)(graphWidth*0.25), graphHeight+30);
        bufferImageSurface.drawLine(50+(int)(graphWidth*0.5), graphHeight+20, 50+(int)(graphWidth*0.5), graphHeight+30);
        bufferImageSurface.drawLine(50+(int)(graphWidth*0.75), graphHeight+20, 50+(int)(graphWidth*0.75), graphHeight+30);

        bufferImageSurface.setColor(new Color(0, 0, 150));
        Font font = new Font("Arial", Font.PLAIN, 12);
        bufferImageSurface.setFont(font);
        bufferImageSurface.drawString(yString, 0, 10);
        bufferImageSurface.drawString(xString, graphWidth+70, graphHeight+50);
        //labeling x-axis
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(xMVal * 0.0), (int)(graphWidth * 0.0 + 30.0), graphHeight+50);
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(xMVal * 0.25), (int)(graphWidth * 0.25 + 30.0), graphHeight+50);
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(xMVal * 0.5), (int)(graphWidth * 0.5 + 30.0), graphHeight+50);
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(xMVal * 0.75), (int)(graphWidth * 0.75 + 30.0), graphHeight+50);
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(xMVal * 1.0), (int)(graphWidth * 1.0 + 30.0), graphHeight+50);
        //labeling y-axis
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(yMVal * 0.25), 0, (int)(graphHeight * 0.75 + 25.0));
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(yMVal * 0.5), 0, (int)(graphHeight * 0.5 + 25.0));
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(yMVal * 0.75), 0, (int)(graphHeight * 0.25 + 25.0));
        bufferImageSurface.drawString(new DecimalFormat("0.0").format(yMVal * 1.0), 0, (int)(graphHeight * 0.0 + 25.0));

        for(i2=0;i2<5;i2++){
          bufferImageSurface.setColor(colorUsed[i2]);
          bufferImageSurface.drawString(rString[i2], graphWidth + 60, i2 * 40 + 15);
          bufferImageSurface.drawString(fString[i2], graphWidth + 60, i2 * 40 + 30);
          if(i2!=dontPaint){
            for(i=1;i<iPos[i2];i++){
              bufferImageSurface.drawLine(i+49, graphHeight-x[i2][i-1]+20, i+50, graphHeight-x[i2][i]+20);
            }
          }
        }
        repaint();
    }

    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Dimension size = getSize();
        Composite origComposite;

        if(bufferImage==null){
          bufferImage = (BufferedImage)createImage(graphWidth + 150, graphHeight + 55);
          bufferImageSurface = bufferImage.createGraphics();
          bufferImageSurface.setBackground(new Color(238, 238, 238));
          bufferImageSurface.clearRect(0, 0, graphWidth + 250, graphHeight + 55);
        }
        g2.drawImage(bufferImage, 0, 0, this);
    }
}

